@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2002 - 2005 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2002 - 2006 Detlev Offenbach <detlev@die-offenbachs.de>
 # Copyright (c) 2000 Phil Thompson <phil@river-bank.demon.co.uk>
 #
 
@@ -259,15 +259,18 @@ class DebugBase(bdb.Bdb):
         
         @param cond condition of the watchpoint to be cleared (string)
         """
-        possibles = bdb.Breakpoint.bplist["Watch", 0]
-        for i in range(0, len(possibles)):
-            b = possibles[i]
-            if b.cond == cond:
-                b.deleteMe()
-                self.breaks["Watch"] -= 1
-                if self.breaks["Watch"] == 0:
-                    del self.breaks["Watch"]
-                break
+        try: 
+            possibles = bdb.Breakpoint.bplist["Watch", 0]
+            for i in range(0, len(possibles)):
+                b = possibles[i]
+                if b.cond == cond:
+                    b.deleteMe()
+                    self.breaks["Watch"] -= 1
+                    if self.breaks["Watch"] == 0:
+                        del self.breaks["Watch"]
+                    break
+        except KeyError:
+            pass
     
     def get_watch(self, cond):
         """
