@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2002 - 2005 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2002 - 2006 Detlev Offenbach <detlev@die-offenbachs.de>
 # Copyright (c) 2000 Phil Thompson <phil@river-bank.demon.co.uk>
 #
 
@@ -29,7 +29,6 @@ from FlexCompleter import Completer
 
 
 DebugClientInstance = None
-debugging = 0
 
 def DebugClientRawInput(prompt="", echo=1):
     """
@@ -279,8 +278,6 @@ class DebugClientBase:
         
         @param line the received line
         """
-        global debugging
-        
         # Remove any newline.
         if line[-1] == '\n':
             line = line[:-1]
@@ -345,7 +342,6 @@ class DebugClientBase:
                 return
 
             if cmd == RequestLoad:
-                debugging = 1
                 self.fncache = {}
                 self.dircache = []
                 sys.argv = []
@@ -391,7 +387,6 @@ class DebugClientBase:
                 return
 
             if cmd == RequestRun:
-                debugging = 0
                 sys.argv = []
                 wd, fn, args = arg.split('|')
                 coding = self.get_coding(fn)
@@ -419,7 +414,7 @@ class DebugClientBase:
                 # we report on all unhandled exceptions
                 sys.excepthook = self.unhandled_exception
                 
-                self.mainThread.tracePython = 0 
+                self.mainThread.tracePython = 0
                 
                 self.debugMod.__dict__['__file__'] = sys.argv[0]
                 sys.modules['__main__'] = self.debugMod
@@ -427,7 +422,6 @@ class DebugClientBase:
                 return
 
             if cmd == RequestCoverage:
-                debugging = 0
                 import PyCoverage
                 sys.argv = []
                 wd, fn, args, erase, exclude = arg.split('@@')
@@ -469,7 +463,6 @@ class DebugClientBase:
                 return
 
             if cmd == RequestProfile:
-                debugging = 0
                 import PyProfile
                 sys.argv = []
                 wd, fn, args, erase = arg.split('|')
@@ -502,7 +495,6 @@ class DebugClientBase:
                 return
 
             if cmd == RequestCyclops:
-                debugging = 0
                 from Cyclops import RunCyclops
                 sys.argv = []
                 wd, fn, args, modfunc, reports = arg.split('|')
@@ -538,8 +530,15 @@ class DebugClientBase:
                 temporary = int(temporary)
 
                 if set:
-                    if cond == 'None':
+                    if cond == 'None' or cond == '':
                         cond = None
+                    else:
+                        try:
+                            compile(cond, '<string>', 'eval')
+                        except:
+                            self.write('%s%s,%d\n' % \
+                                (ResponseBPConditionError, fn, line))
+                            return
                     self.mainThread.set_break(fn, line, temporary, cond)
                 else:
                     self.mainThread.clear_break(fn, line)
@@ -577,6 +576,13 @@ class DebugClientBase:
                 temporary = int(temporary)
 
                 if set:
+                    if not cond.endswith('??created??') and \
+                       not cond.endswith('??changed??'):
+                        try:
+                            compile(cond, '<string>', 'eval')
+                        except:
+                            self.write('%s%s\n' % (ResponseWPConditionError, cond))
+                            return
                     self.mainThread.set_watch(cond, temporary)
                 else:
                     self.mainThread.clear_watch(cond)
@@ -1216,7 +1222,11 @@ class DebugClientBase:
                     exec 'qvar = udict["%s"][%s]' % (rvar[0][:-2], rvar[1])
                 else:
                     qvar = udict[var[-1]]
-                vlist = self.formatQtVariable(qvar)
+                qvtype = unicode(type(qvar))[1:-1].split()[1][1:-1]
+                if qvtype.startswith("PyQt4"):
+                    vlist = self.formatQt4Variable(qvar, qvtype)
+                else:
+                    vlist = self.formatQtVariable(qvar, qvtype)
             else:
                 # format the dictionary found
                 if dictkeys is None:
@@ -1244,80 +1254,177 @@ class DebugClientBase:
         
         self.write('%s%s\n' % (ResponseVariable, str(varlist)))
         
-    def formatQtVariable(self, value):
+    def formatQt4Variable(self, value, vtype):
         """
-        Private method to produce a formated output of a simple Qt type.
+        Private method to produce a formated output of a simple Qt4 type.
         
         @param value variable to be formated
+        @param vtype type of the variable to be formatted (string)
         @return A tuple consisting of a list of formatted variables. Each 
             variable entry is a tuple of three elements, the variable name, 
             its type and value.
         """
-        try:
-            from qt import Qt, QString, QPoint, QRect, QSize, QColor, QCString, \
-                QDate, QDateTime, QTime, QDir, QFile, QFont, QStringList, QUrl
-        except ImportError:
-            return []
-            
-        # XML stuff
-        try:
-            from qtxml import QDomAttr, QDomCharacterData, QDomComment, \
-                QDomElement, QDomText
-            xmlAvail = 1
-        except ImportError:
-            xmlAvail = 0
-            
-        # network stuff
-        try:
-            from qtnetwork import QHostAddress
-            netAvail = 1
-        except ImportError:
-            netAvail = 0
-            
+        qttype = vtype.split('.')[-1]
         varlist = []
-        
-        if isinstance(value, QString):
+        if qttype == 'QString':
             varlist.append(("", "QString", "%s" % unicode(value)))
-        elif isinstance(value, QPoint):
+        elif qttype == 'QStringList':
+            for s in value:
+                varlist.append(("", "QString", "%s" % unicode(s)))
+        elif qttype == 'QChar':
+            varlist.append(("", "QChar", "%s" % unichr(value.unicode())))
+            varlist.append(("", "int", "%d" % value.unicode()))
+        elif qttype == 'QPoint':
             varlist.append(("x", "int", "%d" % value.x()))
             varlist.append(("y", "int", "%d" % value.y()))
-        elif isinstance(value, QRect):
+        elif qttype == 'QPointF':
+            varlist.append(("x", "float", "%g" % value.x()))
+            varlist.append(("y", "float", "%g" % value.y()))
+        elif qttype == 'QRect':
             varlist.append(("x", "int", "%d" % value.x()))
             varlist.append(("y", "int", "%d" % value.y()))
             varlist.append(("width", "int", "%d" % value.width()))
             varlist.append(("height", "int", "%d" % value.height()))
-        elif isinstance(value, QSize):
+        elif qttype == 'QRectF':
+            varlist.append(("x", "float", "%g" % value.x()))
+            varlist.append(("y", "float", "%g" % value.y()))
+            varlist.append(("width", "float", "%g" % value.width()))
+            varlist.append(("height", "float", "%g" % value.height()))
+        elif qttype == 'QSize':
             varlist.append(("width", "int", "%d" % value.width()))
             varlist.append(("height", "int", "%d" % value.height()))
-        elif isinstance(value, QColor):
+        elif qttype == 'QSizeF':
+            varlist.append(("width", "float", "%g" % value.width()))
+            varlist.append(("height", "float", "%g" % value.height()))
+        elif qttype == 'QColor':
             varlist.append(("name", "QString", "%s" % value.name()))
-            r, g, b = value.getRgb()
-            varlist.append(("rgb", "int", "%d, %d, %d" % (r, g, b)))
-            h, s, v = value.getHsv()
-            varlist.append(("hsv", "int", "%d, %d, %d" % (h, s, v)))
-        elif isinstance(value, QCString):
-            varlist.append(("", "QCString", "%s" % unicode(value)))
-        elif isinstance(value, QDate):
-            varlist.append(("", "QDate", "%s" % unicode(value.toString(Qt.LocalDate))))
-        elif isinstance(value, QDateTime):
-            varlist.append(("", "QDateTime", "%s" % unicode(value.toString(Qt.LocalDate))))
-        elif isinstance(value, QTime):
-            varlist.append(("", "QTime", "%s" % unicode(value.toString(Qt.LocalDate))))
-        elif isinstance(value, QDir):
+            r, g, b, a = value.getRgb()
+            varlist.append(("rgb", "int", "%d, %d, %d, %d" % (r, g, b, a)))
+            h, s, v, a = value.getHsv()
+            varlist.append(("hsv", "int", "%d, %d, %d, %d" % (h, s, v, a)))
+            c, m, y, k, a = value.getCmyk()
+            varlist.append(("cmyk", "int", "%d, %d, %d, %d, %d" % (c, m, y, k, a)))
+        elif qttype == 'QDate':
+            varlist.append(("", "QDate", "%s" % unicode(value.toString())))
+        elif qttype == 'QTime':
+            varlist.append(("", "QTime", "%s" % unicode(value.toString())))
+        elif qttype == 'QDateTime':
+            varlist.append(("", "QDateTime", "%s" % unicode(value.toString())))
+        elif qttype == 'QDir':
             varlist.append(("path", "QString", "%s" % unicode(value.path())))
-            varlist.append(("absPath", "QString", "%s" % unicode(value.absPath())))
-        elif isinstance(value, QFile):
-            varlist.append(("name", "QString", "%s" % unicode(value.name())))
-        elif isinstance(value, QFont):
+            varlist.append(("absolutePath", "QString", "%s" % unicode(value.absolutePath())))
+            varlist.append(("canonicalPath", "QString", "%s" % unicode(value.canonicalPath())))
+        elif qttype == 'QFile':
+            varlist.append(("fileName", "QString", "%s" % unicode(value.fileName())))
+        elif qttype == 'QFont':
             varlist.append(("family", "QString", "%s" % unicode(value.family())))
             varlist.append(("pointSize", "int", "%d" % value.pointSize()))
             varlist.append(("weight", "int", "%d" % value.weight()))
             varlist.append(("bold", "bool", "%s" % value.bold()))
             varlist.append(("italic", "bool", "%s" % value.italic()))
-        elif isinstance(value, QStringList):
+        elif qttype == 'QUrl':
+            varlist.append(("url", "QString", "%s" % unicode(value.toString())))
+            varlist.append(("scheme", "QString", "%s" % unicode(value.scheme())))
+            varlist.append(("user", "QString", "%s" % unicode(value.userName())))
+            varlist.append(("password", "QString", "%s" % unicode(value.password())))
+            varlist.append(("host", "QString", "%s" % unicode(value.host())))
+            varlist.append(("port", "int", "%d" % value.port()))
+            varlist.append(("path", "QString", "%s" % unicode(value.path())))
+        elif qttype == 'QModelIndex':
+            varlist.append(("valid", "bool", "%s" % value.isValid()))
+            if value.isValid():
+                varlist.append(("row", "int", "%s" % value.row()))
+                varlist.append(("column", "int", "%s" % value.column()))
+                varlist.append(("internalId", "int", "%s" % value.internalId()))
+        
+        # GUI stuff
+        elif qttype == 'QAction':
+            varlist.append(("name", "QString", "%s" % unicode(value.objectName())))
+            varlist.append(("text", "QString", "%s" % unicode(value.text())))
+            varlist.append(("icon text", "QString", "%s" % unicode(value.iconText())))
+            varlist.append(("tooltip", "QString", "%s" % unicode(value.toolTip())))
+            varlist.append(("whatsthis", "QString", "%s" % unicode(value.whatsThis())))
+            varlist.append(("shortcut", "QString", "%s" % unicode(value.shortcut().toString())))
+        elif qttype == 'QKeySequence':
+            varlist.append(("value", "", "%s" % unicode(value.toString())))
+            
+        # XML stuff
+        elif qttype == 'QDomAttr':
+            varlist.append(("name", "QString", "%s" % unicode(value.name())))
+            varlist.append(("value", "QString", "%s" % unicode(value.value())))
+        elif qttype == 'QDomCharacterData':
+            varlist.append(("data", "QString", "%s" % unicode(value.data())))
+        elif qttype == 'QDomComment':
+            varlist.append(("data", "QString", "%s" % unicode(value.data())))
+        elif qttype == "QDomDocument":
+            varlist.append(("text", "QString", "%s" % unicode(value.toString())))
+        elif qttype == 'QDomElement':
+            varlist.append(("tagName", "QString", "%s" % unicode(value.tagName())))
+            varlist.append(("text", "QString", "%s" % unicode(value.text())))
+        elif qttype == 'QDomText':
+            varlist.append(("data", "QString", "%s" % unicode(value.data())))
+            
+        # Networking stuff
+        elif qttype == 'QHostAddress':
+            varlist.append(("address", "QHostAddress", "%s" % unicode(value.toString())))
+            
+        return varlist
+        
+    def formatQtVariable(self, value, vtype):
+        """
+        Private method to produce a formated output of a simple Qt type.
+        
+        @param value variable to be formated
+        @param vtype type of the variable to be formatted (string)
+        @return A tuple consisting of a list of formatted variables. Each 
+            variable entry is a tuple of three elements, the variable name, 
+            its type and value.
+        """
+        qttype = vtype.split('.')[-1]
+        varlist = []
+        
+        if qttype == "QString":
+            varlist.append(("", "QString", "%s" % unicode(value)))
+        elif qttype == "QPoint":
+            varlist.append(("x", "int", "%d" % value.x()))
+            varlist.append(("y", "int", "%d" % value.y()))
+        elif qttype == "QRect":
+            varlist.append(("x", "int", "%d" % value.x()))
+            varlist.append(("y", "int", "%d" % value.y()))
+            varlist.append(("width", "int", "%d" % value.width()))
+            varlist.append(("height", "int", "%d" % value.height()))
+        elif qttype == "QSize":
+            varlist.append(("width", "int", "%d" % value.width()))
+            varlist.append(("height", "int", "%d" % value.height()))
+        elif qttype == "QColor":
+            varlist.append(("name", "QString", "%s" % value.name()))
+            r, g, b = value.getRgb()
+            varlist.append(("rgb", "int", "%d, %d, %d" % (r, g, b)))
+            h, s, v = value.getHsv()
+            varlist.append(("hsv", "int", "%d, %d, %d" % (h, s, v)))
+        elif qttype == "QCString":
+            varlist.append(("", "QCString", "%s" % unicode(value)))
+        elif qttype == "QDate":
+            varlist.append(("", "QDate", "%s" % unicode(value.toString())))
+        elif qttype == "QDateTime":
+            varlist.append(("", "QDateTime", "%s" % unicode(value.toString())))
+        elif qttype == "QTime":
+            varlist.append(("", "QTime", "%s" % unicode(value.toString())))
+        elif qttype == "QDir":
+            varlist.append(("path", "QString", "%s" % unicode(value.path())))
+            varlist.append(("absPath", "QString", "%s" % unicode(value.absPath())))
+        elif qttype == "QFile":
+            varlist.append(("name", "QString", "%s" % unicode(value.name())))
+        elif qttype == "QFont":
+            varlist.append(("family", "QString", "%s" % unicode(value.family())))
+            varlist.append(("pointSize", "int", "%d" % value.pointSize()))
+            varlist.append(("weight", "int", "%d" % value.weight()))
+            varlist.append(("bold", "bool", "%s" % value.bold()))
+            varlist.append(("italic", "bool", "%s" % value.italic()))
+        elif qttype == "QStringList":
             for s in value:
                 varlist.append(("", "QString", "%s" % unicode(s)))
-        elif isinstance(value, QUrl):
+        elif qttype == "QUrl":
             varlist.append(("host", "QString", "%s" % unicode(value.host())))
             varlist.append(("protocol", "QString", "%s" % unicode(value.protocol())))
             varlist.append(("user", "QString", "%s" % unicode(value.user())))
@@ -1326,22 +1433,37 @@ class DebugClientBase:
             varlist.append(("path", "QString", "%s" % unicode(value.path())))
             varlist.append(("query", "QString", "%s" % unicode(value.query())))
             
+        # GUI stuff
+        elif qttype == 'QAction':
+            from qt import QString
+            varlist.append(("name", "QString", "%s" % unicode(value.name())))
+            varlist.append(("text", "QString", "%s" % unicode(value.menuText())))
+            varlist.append(("icon text", "QString", "%s" % unicode(value.text())))
+            varlist.append(("tooltip", "QString", "%s" % unicode(value.toolTip())))
+            varlist.append(("whatsthis", "QString", "%s" % unicode(value.whatsThis())))
+            varlist.append(("shortcut", "QString", "%s" % unicode(QString(value.accel()))))
+        elif qttype == 'QKeySequence':
+            from qt import QString
+            varlist.append(("value", "", "%s" % unicode(QString(value))))
+            
         # XML stuff
-        elif xmlAvail and isinstance(value, QDomAttr):
+        elif qttype == "QDomAttr":
             varlist.append(("name", "QString", "%s" % unicode(value.name())))
             varlist.append(("value", "QString", "%s" % unicode(value.value())))
-        elif xmlAvail and isinstance(value, QDomCharacterData):
+        elif qttype == "QDomCharacterData":
             varlist.append(("data", "QString", "%s" % unicode(value.data())))
-        elif xmlAvail and isinstance(value, QDomComment):
+        elif qttype == "QDomComment":
             varlist.append(("data", "QString", "%s" % unicode(value.data())))
-        elif xmlAvail and isinstance(value, QDomElement):
+        elif qttype == "QDomDocument":
+            varlist.append(("text", "QString", "%s" % unicode(value.toString())))
+        elif qttype == "QDomElement":
             varlist.append(("tagName", "QString", "%s" % unicode(value.tagName())))
             varlist.append(("text", "QString", "%s" % unicode(value.text())))
-        elif xmlAvail and isinstance(value, QDomText):
+        elif qttype == "QDomText":
             varlist.append(("data", "QString", "%s" % unicode(value.data())))
             
         # Networking stuff
-        elif netAvail and isinstance(value, QHostAddress):
+        elif qttype == "QHostAddress":
             varlist.append(("address", "QHostAddress", "%s" % unicode(value.toString())))
             
         return varlist
@@ -1499,7 +1621,7 @@ class DebugClientBase:
         @param enableTrace flag to enable the tracing function (boolean)
         @param exceptions flag to enable exception reporting of the IDE (boolean)
         """
-        global debugClient, debugging
+        global debugClient
         if host is None:
             host = os.getenv('ERICHOST', 'localhost')
         if port is None:
@@ -1538,9 +1660,6 @@ class DebugClientBase:
         # now start debugging
         if enableTrace:
             self.mainThread.set_trace()
-            debugging = 1
-        else:
-            debugging = 0
         
     def startProgInDebugger(self, progargs, wd = '', host=None, 
             port=None, exceptions=1, tracePython=0, redirect=1):
@@ -1556,9 +1675,6 @@ class DebugClientBase:
         @param tracePython flag to enable tracing into the Python library (boolean)
         @param redirect flag indicating redirection of stdin, stdout and stderr (boolean)
         """
-        global debugging
-        debugging = 1
-        
         if host is None:
             host = os.getenv('ERICHOST', 'localhost')
         if port is None:
@@ -1671,7 +1787,7 @@ class DebugClientBase:
                                          redirect=redirect)
         else:
             try:
-                port = int(sys.argv[1])
+                port = long(sys.argv[1])
             except:
                 port = -1
             try:
