@@ -259,7 +259,14 @@ it trys to execute the lines accumulated so far.
                 return
             
             when RequestContinue
-                DEBUGGER__.context(DEBUGGER__.last_thread).step_continue()
+                special = arg.to_i
+                if special == 0
+                    DEBUGGER__.context(DEBUGGER__.last_thread).step_continue()
+                else
+                    # special == 1 means a continue while doing a step over
+                    # this occurs when an expception is raised doing a step over
+                    DEBUGGER__.context(DEBUGGER__.last_thread).step_over()
+                end
                 @eventExit = true
                 return
             
@@ -272,7 +279,8 @@ it trys to execute the lines accumulated so far.
                 $debugging = true
                 ARGV.clear()
                 wd, fn, args, traceRuby = arg.split("|", -4)
-                ARGV.concat(args.split())
+                @traceRuby = traceRuby.to_i == 1 ? true : false
+                ARGV.concat(eval(args.gsub(/u'/, "'").gsub(/u"/,'"')))
                 $:.insert(0, File.dirname(fn))
                 if wd == ''
                     Dir.chdir($:[0])
@@ -300,7 +308,7 @@ it trys to execute the lines accumulated so far.
                 $debugging = false
                 ARGV.clear()
                 wd, fn, args = arg.split("|", -3)
-                ARGV.concat(args.split())
+                ARGV.concat(eval(args.gsub(/u'/, "'").gsub(/u"/,'"')))
                 $:.insert(0, File.dirname(fn))
                 if wd == ''
                     Dir.chdir($:[0])
@@ -850,21 +858,30 @@ The formated variables list (a list of lists of 3 values) is returned.
             end
             next if matched
             
-            if access
-                if key.to_s == key
-                    key = "'%s'" % key
+            begin
+                if access
+                    if key.to_s == key
+                        key = "'%s'" % key
+                    else
+                        key = key.to_s
+                    end
+                    k = "#{access}[%s]" % key
+                    obj = eval(k, binding_)
                 else
-                    key = key.to_s
+                    obj = eval(key, binding_)
                 end
-                k = "#{access}[%s]" % key
-                obj = eval(k, binding_)
-            else
-                obj = eval(key, binding_)
+            rescue NameError
+                next
             end
             
             if obj or obj.class == NilClass or obj.class == FalseClass
                 otype = obj.class.to_s
-                oval = obj.inspect.gsub(/=>/,":")
+                if obj.inspect.nil?
+                    otype = ""
+                    oval = ""
+                else
+                    oval = obj.inspect.gsub(/=>/,":")
+                end
             else
                 otype = ""
                 oval = obj.inspect
@@ -893,7 +910,7 @@ Private method to extract the address part of an object description.
 @param var object description (String)
 @return the address contained in the object description (String)
 =end
-        m = var.match(/^#<.*?:(.*?) /)
+        m = var.match(/^#<.*?:([^:]*?) /)
         if m
             return m[1]
         else
@@ -994,7 +1011,6 @@ Method used to start the remote debugger.
 @param port portnumber of the debug server (int)
 @param exceptions flag to enable exception reporting of the IDE (boolean)
 @param traceRuby flag to enable tracing into the Ruby library
-      (ignored right now)
 @param redirect flag indicating redirection of stdin, stdout and stderr (boolean)
 =end
         $debugging = true
@@ -1009,6 +1025,8 @@ Method used to start the remote debugger.
         connectDebugger(port, host, redirect) #TCPSocket.gethostbyname(host)[3])
         DEBUGGER__.attach(self)
         
+        @traceRuby = traceRuby
+        
         fn = progargs.shift
         fn = File.expand_path(fn)
         
