@@ -18,6 +18,7 @@ File implementing the real debugger, which is connected to the IDE frontend.
 =end
 
 require 'DebugQuit'
+require 'rbconfig'
 
 class DEBUGGER__
 =begin edoc
@@ -239,20 +240,16 @@ Method to remember the last thread.
             DEBUGGER__.set_last_thread(th)
         end
 
-        def debug_silent_eval(str, binding)
+        def debug_silent_eval(str, binding_)
 =begin edoc
 Method to eval a string without output.
 
 @param str String containing the expression to be evaluated
-@param binding the binding for the evaluation
+@param binding_ the binding for the evaluation
 @return the result of the evaluation
 =end
-            begin
-                val = eval(str, binding)
-                val
-            rescue StandardError, ScriptError
-                nil
-            end
+            val = eval(str, binding_)
+            val
         end
 
         def thnum
@@ -269,7 +266,7 @@ Method returning the thread number of the current thread.
             num
         end
 
-        def debug_command(file, line, id, binding)
+        def debug_command(file, line, id, binding_)
 =begin edoc
 Method to execute the next debug command.
 =end
@@ -281,7 +278,7 @@ Method to execute the next debug command.
                 return
             end
             @frame_pos = 0
-            @frames[0] = [binding, file, line, id]
+            @frames[0] = [binding_, file, line, id]
             stdout.printf_line(@frames)
             MUTEX.unlock
             resume_all
@@ -298,13 +295,13 @@ Method to set the frame position of the current frame.
             end
         end
 
-        def check_break_points(file, pos, binding, id)
+        def check_break_points(file, pos, binding_, id)
 =begin edoc
 Method to check, if the given position contains an active breakpoint.
 
 @param file filename containing the currently executed line (String)
 @param pos line number currently executed (int)
-@param binding current binding object
+@param binding_ current binding object
 @param id (ignored)
 @return flag indicating an active breakpoint (boolean)
 =end
@@ -315,20 +312,26 @@ Method to check, if the given position contains an active breakpoint.
             # bp[4] temporary flag
             # bp[5] condition
             # bp[6] ignore count
+            # bp[7] special condition
+            # bp[8] hash of special values
             return false if break_points.empty?
             for b in break_points
                 if b[0]
                     if b[1] == 0 and b[2] == file and b[3] == pos   # breakpoint
                         # Evaluate condition
                         if b[5]
-                            if debug_silent_eval(b[5], binding)
-                                if b[6] == 0    # ignore count reached
-                                    # Delete once reached if temporary breakpoint
-                                    clear_break_point(file, pos) if b[4]
-                                    return true
-                                else
-                                    b[6] -= 1
+                            begin
+                                if debug_silent_eval(b[5], binding_)
+                                    if b[6] == 0    # ignore count reached
+                                        # Delete once reached if temporary breakpoint
+                                        clear_break_point(file, pos) if b[4]
+                                        return true
+                                    else
+                                        b[6] -= 1
+                                    end
                                 end
+                            rescue StandardError, ScriptError
+                                nil
                             end
                         else
                             if b[6] == 0    # ignore count reached
@@ -340,13 +343,53 @@ Method to check, if the given position contains an active breakpoint.
                             end
                         end
                     elsif b[1] == 1                                 # watchpoint
-                        if debug_silent_eval(b[2], binding)
-                            if b[6] == 0    # ignore count reached
-                                # Delete once reached if temporary breakpoint
-                                clear_watch_point(b[2]) if b[4]
-                                return true
+                        begin
+                            bd = @frame_pos
+                            val = debug_silent_eval(b[5], binding_)
+                            if b[7].length() > 0
+                                if b[7] == "??created??"
+                                    if b[8][bd][0] == false
+                                        b[8][bd][0] = true
+                                        b[8][bd][1] = val
+                                        return true
+                                    else
+                                        next
+                                    end
+                                end
+                                b[8][bd][0] = true
+                                if b[7] == "??changed??"
+                                    if b[8][bd][1] != val
+                                        b[8][bd][1] = val
+                                        if b[8][bd][2] > 0
+                                            b[8][bd][2] -= 1
+                                            next
+                                        else
+                                            return true
+                                        end
+                                    else
+                                        next
+                                    end
+                                end
+                                next
+                            end
+                            if val
+                                if b[6] == 0    # ignore count reached
+                                    # Delete once reached if temporary breakpoint
+                                    clear_watch_point(b[2]) if b[4]
+                                    return true
+                                else
+                                    b[6] -= 1
+                                end
+                            end
+                        rescue StandardError, ScriptError
+                            if b[7].length() > 0
+                                if b[8][bd]
+                                    b[8][bd][0] = false
+                                else
+                                    b[8][bd] = [false, nil, b[6]]
+                                end
                             else
-                                b[6] -= 1
+                                val = nil
                             end
                         end
                     end
@@ -439,7 +482,12 @@ Method to add a watchpoint.
 @param cond condition of the watchpoint (String)
 @param temp flag indicating a temporary watchpoint (boolean)
 =end
-            break_points.push [true, 1, cond, 0, temp, "", 0]
+            co1, co2 = cond.split()
+            if co2 == "??created??" or co2 == "??changed??"
+                break_points.push [true, 1, cond, 0, temp, co1, 0, co2, {}]
+            else
+                break_points.push [true, 1, cond, 0, temp, cond, 0, "", {}]
+            end
         end
     
         def delete_watch_point(cond)
@@ -483,14 +531,14 @@ Method to set the ignore count of a watchpoint.
             end
         end
     
-        def excn_handle(file, line, id, binding)
+        def excn_handle(file, line, id, binding_)
 =begin edoc
 Method to handle an exception
 
 @param file filename containing the currently executed line (String)
 @param pos line number currently executed (int)
 @param id (ignored)
-@param binding current binding object
+@param binding_ current binding object
 =end
             if $!.class <= SystemExit
                 set_trace_func nil
@@ -510,7 +558,7 @@ Method to handle an exception
                 end
                 stdout.printf_excn(exclist)
             end
-            debug_command(file, line, id, binding)
+            debug_command(file, line, id, binding_)
         end
 
         def skip_it?(file)
@@ -526,6 +574,12 @@ debugger that are called from the application being debugged.
             if file =~ /\(eval\)/
                 return true
             end
+           
+            if not traceRuby? and
+               (file =~ /#{Config::CONFIG['sitelibdir']}/ or
+                file =~ /#{Config::CONFIG['rubylibdir']}/)
+                return true
+            end
             
             if ["AsyncFile.rb", "AsyncIO.rb", "Config.rb", "DebugClient.rb",
                 "DebugClientBaseModule.rb", "DebugClientCapabilities.rb",
@@ -536,7 +590,7 @@ debugger that are called from the application being debugged.
             return false
         end
     
-        def trace_func(event, file, line, id, binding, klass)
+        def trace_func(event, file, line, id, binding_, klass)
 =begin edoc
 Method executed by the tracing facility.
 
@@ -544,7 +598,7 @@ Method executed by the tracing facility.
 @param file the name of the file being traced (String)
 @param line the line number being traced (int)
 @param id object id
-@param binding a binding object
+@param binding_ a binding object
 @param klass name of a class
 =end
             context(Thread.current).check_suspend
@@ -555,13 +609,13 @@ Method executed by the tracing facility.
                     frame_set_pos(file, line)
                     
                 when 'call'
-                    @frames.unshift [binding, file, line, id]
+                    @frames.unshift [binding_, file, line, id]
                 
                 when 'c-call'
                     frame_set_pos(file, line)
             
                 when 'class'
-                    @frames.unshift [binding, file, line, id]
+                    @frames.unshift [binding_, file, line, id]
                     
                 when 'return', 'end'
                     @frames.shift
@@ -570,7 +624,7 @@ Method executed by the tracing facility.
                     @frames.shift
         
                 when 'raise' 
-                    excn_handle(file, line, id, binding)
+                    excn_handle(file, line, id, binding_)
                     
                 end
                 @last_file = file
@@ -592,24 +646,24 @@ Method executed by the tracing facility.
                 else
                     # nothing to do. skipped.
                 end
-                if check_break_points(file, line, binding, id) or @stop_next == 0 
+                if check_break_points(file, line, binding_, id) or @stop_next == 0 
                     @no_step = nil
                     suspend_all
-                    debug_command(file, line, id, binding)
+                    debug_command(file, line, id, binding_)
                 end
     
             when 'call'
-                @frames.unshift [binding, file, line, id]
-                if check_break_points(file, id.id2name, binding, id) or
-                    check_break_points(klass.to_s, id.id2name, binding, id)
+                @frames.unshift [binding_, file, line, id]
+                if check_break_points(file, id.id2name, binding_, id) or
+                    check_break_points(klass.to_s, id.id2name, binding_, id)
                     suspend_all
-                    debug_command(file, line, id, binding)
+                    debug_command(file, line, id, binding_)
                 end
     
             when 'c-call'
                 frame_set_pos(file, line)
                 if id == :require and klass == Kernel
-                    @frames.unshift [binding, file, line, id]
+                    @frames.unshift [binding_, file, line, id]
                 else
                     frame_set_pos(file, line)
                 end
@@ -624,7 +678,7 @@ Method executed by the tracing facility.
                 end
     
             when 'class'
-                @frames.unshift [binding, file, line, id]
+                @frames.unshift [binding_, file, line, id]
     
             when 'return', 'end'
                 if @frames.size == @finish_pos
@@ -639,7 +693,7 @@ Method executed by the tracing facility.
             when 'raise'
                 @no_step = nil
                 @stop_next = 0        # break here before leaving...
-                excn_handle(file, line, id, binding)
+                excn_handle(file, line, id, binding_)
     
             end
             @last_file = file
@@ -902,12 +956,12 @@ Method to create a thread list.
             @thread_list = hash
         end
 
-        def debug_thread_info(input, binding)
+        def debug_thread_info(input, binding_)
 =begin edoc
 Method handling the thread related debug commands.
 
 @param input debug command (String)
-@param binding reference to the binding object
+@param binding_ reference to the binding object
 =end
             case input
             when /^l(?:ist)?/
@@ -969,6 +1023,13 @@ Method calling the main function polling for an event sent by the IDE.
 =end
             @client.eventPoll
         end
+        
+        def traceRuby?
+=begin edoc
+Method to check, if we should trace into the Ruby interpreter libraries.
+=end
+            @client.traceRuby?
+        end
     end
 
 
@@ -986,6 +1047,13 @@ Method calling the main function polling for an event sent by the IDE.
 =end
             DEBUGGER__.eventPoll
         end
+        
+        def traceRuby?
+=begin edoc
+Method to check, if we should trace into the Ruby interpreter libraries.
+=end
+            DEBUGGER__.traceRuby?
+        end
     end
 
     require 'DebugProtocol'
@@ -1016,6 +1084,13 @@ Method calling the main function polling for an event sent by the IDE.
 =end
             @debugger.eventPoll()
         end
+        
+        def traceRuby?
+=begin edoc
+Method to check, if we should trace into the Ruby interpreter libraries.
+=end
+            @debugger.traceRuby
+        end
 
         def printf( *args )
 =begin edoc
