@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2002 - 2005 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2002 - 2006 Detlev Offenbach <detlev@die-offenbachs.de>
 # Copyright (c) 2000 - 2002 Phil Thompson <phil@river-bank.demon.co.uk>
 #
 
@@ -956,11 +956,23 @@ class DebugServer(QServerSocket):
                     self.emit(PYSIGNAL('clientClearBreak'),(fn, lineno))
                     continue
                     
+                if resp == ResponseBPConditionError:
+                    fn, lineno = line[eoc:-1].split(',')
+                    lineno = int(lineno)
+                    fn = self.translate(fn, 1)
+                    self.emit(PYSIGNAL('clientBreakConditionError'),(fn, lineno))
+                    continue
+                    
                 if resp == ResponseClearWatch:
                     cond = line[eoc:-1]
                     self.emit(PYSIGNAL('clientClearWatch'),(cond,))
                     continue
                     
+                if resp == ResponseWPConditionError:
+                    cond = line[eoc:-1]
+                    self.emit(PYSIGNAL('clientWatchConditionError'),(cond,))
+                    continue
+                    
                 if resp == ResponseRaw:
                     prompt, echo = eval(line[eoc:-1])
                     self.emit(PYSIGNAL('clientRawInput'),(prompt, echo))
