@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2002 - 2005 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2002 - 2006 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -86,6 +86,8 @@ class DebugUI(QObject):
         self.connect(dbs,PYSIGNAL('clientVariables'),self.handleClientVariables)
         self.connect(dbs,PYSIGNAL('clientVariable'),self.handleClientVariable)
         self.connect(dbs,PYSIGNAL('clientClearBreak'),self.handleClientClearBreak)
+        self.connect(dbs,PYSIGNAL('clientBreakConditionError'),
+            self.handleClientBreakConditionError)
         self.connect(dbs,PYSIGNAL('passiveDebugStarted'),self.handlePassiveDebugStarted)
         self.connect(dbs,PYSIGNAL('cyclopsError'),self.handleCyclopsError)
         
@@ -978,6 +980,26 @@ class DebugUI(QObject):
         if editor is not None:
             editor.clearBreakpoint(lineno)
         
+    def handleClientBreakConditionError(self, filename, lineno):
+        """
+        Private method to handle a condition error of a breakpoint.
+        
+        @param filename filename of the breakpoint
+        @param lineno linenumber of the breakpoint
+        """
+        KQMessageBox.critical(None,
+            self.trUtf8("Breakpoint Condition Error"),
+            self.trUtf8("""<p>The condition of the breakpoint <b>%1, %2</b>"""
+                        """ contains a syntax error.</p>""")\
+                        .arg(filename).arg(lineno),
+            self.trUtf8("&OK"),
+            QString.null,
+            QString.null,
+            0, -1)
+        editor = self.viewmanager.getOpenEditor(filename)
+        if editor is not None:
+            editor.handleEditBreakpoint(0, lineno)
+        
     def handleConfigFilters(self):
         """
         Private slot for displaying the variables filter configuration dialog.
